*************************************************************
* Program Name = FLPC.SAS                                   *
* Program Purpose = To perform Fisher-Lancaster analysis    *
*     on HSAB.DAT which uses the FL.MAC macro               *
*************************************************************;
options     mprint ;
DATA ONE;
INFILE 'B:HSAB.DAT' MISSOVER;
   ********if running on hard drive -> change INFILE statement
           to reflect current directory *****************************;
   INPUT CHURCH X1 X2 X3 X4;
   DUM = 1 ;
 /*********************************************************
This macro performs the Fisher-Lancaster scaling.
It must be called once per categorical valued predictor
 variable.
NUM starts out with a value of 1 and must be incremented
 by 1 for each additional categorical variable.
DATA designates the SAS dataset containing the data.
GROUP identifies the variable identifying group membership.
VAR identifies the categorical valued variable to be
 scaled.
NGROUPS is the value for the number of criterion groups.
NCATS IS THE NUMBER OF CATEGORIES IN THE VARIABLE TO BE SCALED.
NCATS2 is the value of 2 x the number of categories in the
 variable to be scaled.
It prints out a cross-classification table for each categorical
 valued variable against the grouping variable (Pages 1 & 3)
It also prints out the scaling coefficients for each categorical
 valued variable (WP) and the eigenvalues (LAM).
On each call it generates F-L scaled values for the designated
 categorical valued variable for the first two eigenvalues
 (SCALEAi and SCALEBi respectively).  These are appended to
 the original data set.
************************************************************/


%MACRO FL(NUM,DATA=,GROUP=,VAR=,NGROUPS=,NCATS2=,NCATS=);
title "&num  &DATA  &GROUP  &VAR &NGROUPS  &NCATS2  &NCATS" ;
PROC FREQ DATA=&DATA;
  TABLES &VAR*&GROUP/SPARSE OUT=STUFF;
PROC IML; /*---Start of MATRIX/IML Translation---*/
RESET AUTONAME
/* CHECK: Option ERRMAX not available in IML */;

START MAIN;
  USE STUFF (KEEP=COUNT);
  READ ALL INTO RDTA ;
  VX=SHAPE(RDTA,&NCATS,&NGROUPS);
  PRINT VX;
  VCS= VX[+,];
  VRS= VX[,+];
  VS= VX[+,+];
  D= DIAG( RECIP(VCS));
  VXX=VX*D*VX`;
  VRSRS=VRS*VRS`;
  VXX=VXX-VRSRS/VS;
  Q=VXX/ SQRT(VRSRS);
  PRINT Q;
  CALL EIGEN( LAM, VEC, Q);
  QVEC= VEC[,{1}:{2}];
  AV=QVEC/( SQRT(VRS)|| SQRT(VRS));

  AVSQ=AV##{2};
  DELSQ=VS/(VRS`*AVSQ);
  DEL= SQRT(DELSQ);
  WP=AV;
  WP[,{1}]= WP[,{1}]* DEL[{1},{1}];
  WP[,{2}]= WP[,{2}]* DEL[{1},{2}];
  WP=WP`;
  W=SHAPE(WP,1,&NCATS2);
  PRINT LAM, WP , W;
  _TMP_ROW = 'ROW1    ' : compress('ROW'+char(nrow(W)));
  CLOSE TEMP;
  CREATE TEMP ( RENAME=(_TMP_ROW=ROW  )) FROM W [ROWNAME=_TMP_ROW ];
  APPEND FROM W [ROWNAME=_TMP_ROW];
  STOP;

FINISH MAIN;

RUN MAIN;  /*---End of MATRIX/IML Translation---*/
quit;
DATA TEMP;
  SET TEMP;
  DUM=1;
DATA &DATA(DROP=COL1-COL&NCATS2);
  MERGE &DATA TEMP;
    BY DUM;
  ARRAY WGT (I) COL1-COL&NCATS2;
    I=&VAR;
    SCALEA&NUM=WGT;
    I=&NCATS2/2+&VAR;
    SCALEB&NUM=WGT;
%mend  FL  ;
%FL(1,DATA=ONE,GROUP=CHURCH,VAR=X3,NGROUPS=3,NCATS2=10,NCATS=5)
%FL(2,DATA=ONE,GROUP=CHURCH,VAR=X4,NGROUPS=3,NCATS2=8,NCATS=4)
PROC PRINT;
RUN;
