*************************************************************
* Program Name = YAOPC.SAS                                  *
* Program Purpose = To calculate T-sq and F-statistic       *
*    for contrasts involving non-equal covariance matrices  *
*    (PC-version using PROC IML)                            *
*************************************************************;
OPTIONS PAGESIZE=60;
DATA ONE;

*infile baumann3g2v.sas missover;
   ********if running on hard drive -> change INFILE statement
           to reflect current directory *****************************;
 
input grp edt drp;
cards;
1	4	43
1	4	34
1	4	45
1	3	39
1	8	40
1	1	27
1	7	46
1	7	39
1	9	31
1	6	39
1	4	40
1	12	52
1	14	53
1	12	53
1	7	41
1	5	41
1	9	46
1	13	52
1	11	55
1	5	36
1	11	50
1	15	54
2	5	34
2	9	36
2	5	42
2	7	37
2	4	44
2	9	49
2	3	38
2	4	38
2	2	38
2	5	50
2	7	31
2	8	49
2	10	54
2	9	52
2	12	50
2	5	35
2	8	36
2	12	46
2	4	42
2	8	47
2	6	39
2	5	38
3	6	27
3	6	36
3	5	51
3	5	51
3	0	50
3	6	55
3	6	52
3	11	48
3	6	53
3	8	45
3	8	47
3	3	51
3	7	30
3	7	50
3	6	55
3	9	48
3	7	52	
3	6	46
3	7	36
3	6	45
3	6	49
3	6	49
DATA TWO; SET ONE;
IF GRP=1 OR GRP=3;
PROC IML; /*---Start of MATRIX/IML Translation---*/
*RESET AUTONAME ;

START MAIN;
  USE _LAST_ ;
  READ ALL INTO X ;
  PRINT X;
  *State the first group size;
  N1={22};
  *State the second group size;
  N2={22};
  *State the number of outcome variables;
  P={2};
  *identify the location of the outcome variables i.e., columns 2 through 3;
  X1= X[{1}:N1,{2}:{3}];
  X2= X[(N1+{1}):(N1+N2),{2}:{3}];
  PRINT X1, X2;
  SUM1= X1[+,];
  MEAN1=(SUM1)/N1;
  SUM2= X2[+,];
  MEAN2=(SUM2)/N2;
  E1= J(N1,N1,{1});
  E2= J(N2,N2,{1});
  S1={1}/(N1-{1})*X1`*( I(N1)-({1}/N1)*E1)*X1;
  S2={1}/(N2-{1})*X2`*( I(N2)-({1}/N2)*E2)*X2;
  SPUN=S1/N1+S2/N2;
  TSQV=(MEAN1-MEAN2)* INV(SPUN)*(MEAN1-MEAN2)`;
 
  W1=S1/N1;
  W2=S2/N2;
  W=W1+W2;
  WI= INV(W);
  WIN1=WI*W1*WI;
  WIN2=WI*W2*WI;
  MEAN=MEAN1-MEAN2;
  R=({1}/(N1-{1}))*((MEAN*WIN1*MEAN`/TSQV)**{2})
 +({1}/(N2-{1}))*((MEAN*WIN2*MEAN`/TSQV)**{2});
  F={1}/R;
 
  C=F-P+{1};
  FV=C/(P*(F))*TSQV;
  PII={1}- PROBF(FV,P,C);
 
  STOP;

FINISH MAIN;


RUN MAIN;  /*---End of MATRIX/IML Translation---*/

PRINT MEAN1 MEAN2 S1 S2 TSQV FV P C PII;


*RUN;   * (This line may be omitted);
